package PlayBot::sites;

use strict;
use warnings;
use utf8;

use Module::Pluggable sub_name => 'sites', search_path => ['PlayBot::sites'], require => 1;
use Storable qw(dclone);

use PlayBot::utils::db;
use PlayBot::utils::db::chan;
use PlayBot::utils::print;
use PlayBot::commands::parser;

our $irc;
our $log;
our $regex;


sub parse {
	my ($kernel, $user, $chan, $msg, $playlist) = @_;
	my ($nick,$mask) = split(/!/,$user);

    my $chan_conf = PlayBot::utils::db::chan->new($chan);

    my %content;

    my $matching_url;
    my @matching_tags;

    # parsing
    foreach my $site (__PACKAGE__->sites)
    {
        not grep { $site eq "PlayBot::sites::$_" } @{ $chan_conf->sites } and next;

        my @args;
        if (@args = ($msg =~ $site->regex))
        {
		    eval { %content = $site->get(@args) };
            $matching_url = $&;
            $content{playlist} = 0;
            last;
        }
        elsif ($site->can('regex_playlist') and @args = ($msg =~ $site->regex_playlist))
        {
            eval { %content = $site->get_playlist(@args) };
            $matching_url = $&;
            $content{playlist} = 1;
            last;
        }
    }

    # something goes wrong ?
    if ($@) {
        $log->warning ($@);
        return;
    }

    my $id;
    # if we get a new content, we must save it
    if (%content) {
        ($id, @matching_tags) = insert_content($kernel, $nick, $chan, dclone(\%content), $msg, $playlist);

        if (not $playlist and $content{playlist})
        {
            foreach my $url (@{ $content{urls} })
            {
                my $new_msg = $msg;
                $new_msg =~ s/\Q$matching_url\E/$url/;
                parse($kernel, $user, $chan, $new_msg, $id);
            }
        }
    }

    return (
        $matching_url,
        @matching_tags
    );
}

sub insert_content
{
    my ($kernel, $nick, $chan, $content, $msg, $playlist) = @_;
    my $dbh = PlayBot::utils::db::main_session();
    my $id;

    # we assume it's a new content
    my $new = 1;

	# insertion de la vidéo dans la bdd
    eval {
        my $sth = $dbh->prepare('
            INSERT INTO playbot (type, url, external_id, sender, title, duration, playlist)
            VALUES (?,?,?,?,?,?,?)
        ');
		$log->error("Couldn't prepare querie; aborting") unless (defined $sth);

		$sth->execute(
            $content->{site},
            $content->{url},
            $content->{external_id},
            $content->{author},
            $content->{title},
            $content->{duration},
            $content->{playlist},
        );
    };
    if ($@) {
        # seems to be already present in database 
        $new = 0;

        my $sth = $dbh->prepare('
            UPDATE playbot SET
            sender = ?,
            title = ?,
            duration = ?,
            external_id = ?
            WHERE url = ?
        ');
		$log->error("Couldn't prepare querie; aborting") unless (defined $sth);

		$sth->execute(
            $content->{author},
            $content->{title},
            $content->{duration},
            $content->{external_id},
            $content->{url},
        );
    }

	# sélection de l'id de la vidéo insérée
	my $sth = $dbh->prepare('SELECT id FROM playbot WHERE url = ?');
	$log->error("Couldn't prepare querie; aborting") unless (defined $sth);

	$sth->execute($content->{url})
        or $log->error("Couldn't finish transaction: " . $dbh->errstr);

	$id = $sth->fetch->[0];

    if (defined($playlist))
    {
        # save track in the playlist
        $sth = $dbh->prepare('
            INSERT INTO playbot_playlist_content_association (playlist_id, content_id)
            VALUES (?,?)');
	    $log->error("Couldn't prepare querie; aborting") unless (defined $sth);

        eval {
            $sth->execute($playlist, $id);
            $dbh->commit;
        };
        if ($@)
        {
            # the association already exists
            $dbh->rollback;
        }
    }
    else
    {
        # insertion du chan
        $sth = $dbh->prepare('
            INSERT INTO playbot_chan (content, chan, sender_irc)
            VALUES (?,?,?)');
	    $log->error("Couldn't prepare querie; aborting") unless (defined $sth);

        $sth->execute($id, $chan, $nick)
            or $log->error("Couldn't finish transaction: " . $dbh->errstr);

        $dbh->commit;
    }

    my @matching_tags = PlayBot::commands::parser::tag($msg, $chan, $id);

    my @tags;
    # get tags
    $sth = $dbh->prepare("select tag
        from playbot_tags
        where id = ?
        ");
    $sth->execute($id);

    while (my $data = $sth->fetch) {
        my $tag = $data->[0];
        $tag =~ s/([a-zA-Z0-9_-]+)/#$1/;
        push @tags, $tag;
    }
    $dbh->commit;

    if ($new) {
        # schedule download
        $kernel->post(downloader => ddl => $id, $content->{url});

        $content->{id} = '+'.$id;
    }
    else {
        $content->{id} = $id;
    }
    $content->{tags} = \@tags;
    delete $content->{url};
    delete $content->{external_id};

    if (not $playlist)
    {
        # message sur irc
	    $irc->yield(privmsg => $chan => PlayBot::utils::print::print($content));
    }

    return $content->{id}, @matching_tags;
}

1;
